use {
    digest::Digest,
    rustls::Certificate,
    sha2::Sha256,
    std::{fmt::Write, io::Read},
    x509_parser::prelude::*,
};

mod error;
pub use error::Error;

/// Creates an sha256 fingerprint for a certificate
pub trait GetFingerprint {
    type Error;

    fn fingerprint(&self) -> Result<Fingerprint, Self::Error>;
}

pub struct Fingerprint {
    pub names: Vec<String>,
    pub fingerprint: String,
}

impl GetFingerprint for Certificate {
    type Error = Error;

    fn fingerprint(&self) -> Result<Fingerprint, Self::Error> {
        let (_, pk) = X509Certificate::from_der(self.as_ref())?;
        let subject = pk.subject();
        let mut names = vec![];
        subject.iter_common_name().for_each(|n| {
            let mut val = n.attr_value().data;
            let mut name = String::new();
            if let Ok(_) = val.read_to_string(&mut name) {
                names.push(name);
            }
        });
        if !pk.validity().is_valid() {
            return Err(Error::InvalidForDate);
        }
        let key = pk.public_key().subject_public_key.as_ref();
        let mut hasher = Sha256::new();
        hasher.update(key);
        let res = hasher.finalize();
        let mut s = String::with_capacity(res.len());
        for c in res {
            write!(s, "{c:02x}")?;
        }
        Ok(Fingerprint {
            names,
            fingerprint: s,
        })
    }
}
